@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 
-# Copyright (c) 2016-2019, Magnus Edenhill
+# Copyright (c) 2016-2021, Magnus Edenhill
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,7 @@
 from trivup import trivup
 from trivup.apps.KerberosKdcApp import KerberosKdcApp
 from trivup.apps.SslApp import SslApp
+from trivup.apps.OauthbearerOIDCApp import OauthbearerOIDCApp
 
 from string import Template
 import contextlib
@@ -218,10 +219,7 @@ class KafkaBrokerApp (trivup.App):
         self.dbg('Advertised Listeners: %s' %
                  self.conf['advertised.listeners'])
 
-        if not self.kraft:
-            # SimpleAclAuthorizer doesn't work with KRaft, for some reason.
-            # Will investigate later.
-            conf_blob.append('authorizer.class.name=kafka.security.auth.SimpleAclAuthorizer')   # noqa: E501
+        self._add_simple_authorizer(conf_blob)
 
         if len(sasl_mechs) > 0:
             self.dbg('SASL mechanisms: %s' % sasl_mechs)
@@ -255,7 +253,7 @@ class KafkaBrokerApp (trivup.App):
                                              (u, p))
                             # SCRAM users are set up using kafka-configs.sh
                             self.post_start_cmds.append((
-                                '%s --bootstrap-server %s '
+                                'JMX_PORT="" %s --bootstrap-server %s '
                                 '--alter --add-config '
                                 '\'%s=[iterations=4096,password=%s]\' '
                                 '--entity-type users --entity-name \'%s\'') % (
@@ -280,7 +278,7 @@ class KafkaBrokerApp (trivup.App):
                     # hostname ("admin" rather than "admin/localhost")
                     # to a local user.
                     # This is not compatible with "admin/localhost" principals.
-                    conf_blob.append('authorizer.class.name=kafka.security.auth.SimpleAclAuthorizer')  # noqa: E501
+                    self._add_simple_authorizer(conf_blob)
                     conf_blob.append('allow.everyone.if.no.acl.found=true')
                     conf_blob.append('sasl.kerberos.principal.to.local.rules=RULE:[1:admin](.*)s/^.*/admin/')  # noqa: E501
 
@@ -298,18 +296,32 @@ class KafkaBrokerApp (trivup.App):
                 jaas_blob.append('principal="%s";' % self.kerberos_principal)
 
             if 'OAUTHBEARER' in sasl_mechs:
-                # Use the unsecure JSON web token.
-                # Client should be configured with
-                # 'sasl.oauthbearer.config=scope=requiredScope principal=admin'
-                # Change requiredScope to something else to trigger auth error.
-                conf_blob.append('super.users=User:admin')
-                conf_blob.append('allow.everyone.if.no.acl.found=true')
-                conf_blob.append('authorizer.class.name=kafka.security.auth.SimpleAclAuthorizer')  # noqa: E501
-                jaas_blob.append('org.apache.kafka.common.security.oauthbearer.OAuthBearerLoginModule required')  # noqa: E501
-                jaas_blob.append('  unsecuredLoginLifetimeSeconds="3600"')
-                jaas_blob.append('  unsecuredLoginStringClaim_sub="admin"')
-                jaas_blob.append('  unsecuredValidatorRequiredScope="requiredScope"')  # noqa: E501
-                jaas_blob.append(';')
+                oidcapp = self.cluster.find_app(OauthbearerOIDCApp)
+                if oidcapp is not None:
+                    assert self.version >= [3, 1, 0], "OIDC requires Apache Kafka 3.1 or later"
+                    # Use the OIDC method.
+                    conf_blob.append('listener.name.sasl_plaintext.oauthbearer.sasl.server.callback.handler.class=org.apache.kafka.common.security.oauthbearer.secured.OAuthBearerValidatorCallbackHandler')
+                    conf_blob.append('listener.name.sasl_plaintext.oauthbearer.sasl.oauthbearer.jwks.endpoint.url=%s' % oidcapp.conf['jwks_url'])
+                    conf_blob.append('listener.name.sasl_plaintext.oauthbearer.sasl.oauthbearer.scope.claim.name=scp')
+                    conf_blob.append('listener.name.sasl_plaintext.oauthbearer.sasl.jaas.config=org.apache.kafka.common.security.oauthbearer.OAuthBearerLoginModule required unsecuredLoginStringClaim_sub="unused";')
+                    conf_blob.append('listener.name.sasl_plaintext.oauthbearer.sasl.oauthbearer.expected.audience=api://default')
+                    conf_blob.append('security.inter.broker.protocol=PLAINTEXT')
+                    conf_blob.append('sasl.enabled.mechanisms=OAUTHBEARER')
+                else:
+                    # Use the unsecure JSON web token.
+                    # Client should be configured with
+                    # 'sasl.oauthbearer.config=scope=requiredScope principal=
+                    # admin'
+                    # Change requiredScope to something else to trigger auth
+                    # error.
+                    conf_blob.append('super.users=User:admin')
+                    conf_blob.append('allow.everyone.if.no.acl.found=true')
+                    self._add_simple_authorizer(conf_blob)
+                    jaas_blob.append('org.apache.kafka.common.security.oauthbearer.OAuthBearerLoginModule required')  # noqa: E501
+                    jaas_blob.append('  unsecuredLoginLifetimeSeconds="3600"')
+                    jaas_blob.append('  unsecuredLoginStringClaim_sub="admin"')
+                    jaas_blob.append('  unsecuredValidatorRequiredScope="requiredScope"')  # noqa: E501
+                    jaas_blob.append(';')
 
             jaas_blob.append('};\n')
             self.conf['jaas_file'] = self.create_file('jaas_broker.conf',
@@ -443,3 +455,11 @@ class KafkaBrokerApp (trivup.App):
         # easily called.
         self.env_add('PATH', os.environ.get('PATH') + ':' +
                      os.path.join(destdir, 'bin'), append=False)
+
+    def _add_simple_authorizer(self, conf_blob):
+        # Kafka removed SimpleAclAuthorizer class in v3.0.0
+        # https://github.com/apache/kafka/commit/976e78e405d57943b989ac487b7f49119b0f4af4#diff-e0ccf1b5c964d2c303b6a69a8b8b67df5a6bfbae8aa514f580d353c4c6bf8e36
+        if self.version[0] >= 3:
+            conf_blob.append('authorizer.class.name=kafka.security.authorizer.AclAuthorizer')   # noqa: E501
+        else:
+            conf_blob.append('authorizer.class.name=kafka.security.auth.SimpleAclAuthorizer')   # noqa: E501
